"use strict";

exports.__esModule = true;
exports.createStyles = createStyles;
exports.compileStyles = compileStyles;

var _cssToReactNative = /*#__PURE__*/_interopRequireDefault( /*#__PURE__*/require("css-to-react-native"));

var React = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("react"));

var RN = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("react-native"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const __reactCreateElement__ = React.createElement;
const emptyObj = {};

function createStyles(options = emptyObj) {
  var _options$themes, _options$tokens, _Object$values$;

  const themes = (_options$themes = options.themes) !== null && _options$themes !== void 0 ? _options$themes : emptyObj;
  const initialTokens = (_options$tokens = options.tokens) !== null && _options$tokens !== void 0 ? _options$tokens : {};
  let tokens = initialTokens;
  if (options.themes) mergeTokens(tokens, (_Object$values$ = Object.values(themes)[0]) !== null && _Object$values$ !== void 0 ? _Object$values$ : {});
  const generated = new Map();

  if (process.env.NODE_ENV !== "production") {
    const set_ = generated.set.bind(generated);

    generated.set = (key, styles) => set_(key, Object.freeze(styles));
  }

  function cls(literals, ...placeholders) {
    const styles = compileStyles(compileLiterals(literals, ...placeholders), tokens);

    if (process.env.NODE_ENV !== "production") {
      return Object.freeze(styles);
    }

    return styles;
  }

  const styles = Object.assign(function styles(styleMap) {
    const compiledStyleMap = new Map();
    let styleKey;
    /* istanbul ignore next */

    for (styleKey in styleMap) {
      var _styleMap$styleKey;

      compiledStyleMap.set(styleKey, compileStyles((_styleMap$styleKey = styleMap[styleKey]) !== null && _styleMap$styleKey !== void 0 ? _styleMap$styleKey : emptyObj, tokens));
    } // style('text', {})


    function style(...args) {
      var _compiledStyleMap$get;

      const key = JSON.stringify(args);
      const rules = generated.get(key);
      if (rules) return rules;
      const numArgs = args.length;
      const sheet = (_compiledStyleMap$get = compiledStyleMap.get("default")) !== null && _compiledStyleMap$get !== void 0 ? _compiledStyleMap$get : {};

      if (numArgs === 1 && typeof args[0] === "string") {
        Object.assign(sheet, compiledStyleMap.get(args[0]));
      } else if (numArgs > 0) {
        let i = 0;
        let arg;

        for (; i < numArgs; i++) {
          arg = args[i];

          if (typeof arg === "string") {
            Object.assign(sheet, compiledStyleMap.get(arg));
          } else if (typeof arg === "object") {
            for (const key in arg) if (arg[key]) Object.assign(sheet, compiledStyleMap.get(key));
          }
        }
      }

      generated.set(key, sheet);
      return sheet;
    }

    style.styles = styleMap;
    return style;
  }, {
    one(literals, ...placeholders) {
      const styles = compileStyles(compileLiterals(literals, ...placeholders), tokens);
      return Object.assign(function oneStyle(createStyle) {
        if (!createStyle && createStyle !== void 0) return emptyObj;
        return styles;
      }, {
        styles
      });
    },

    cls,

    lazy(lazyFn) {
      const cache = new Map();
      return function (value) {
        if (!value) return emptyObj;
        const key = typeof value === "object" ? JSON.stringify(value) : value;
        const cached = cache.get(key);
        if (cached) return cached;
        let styles = compileStyles(lazyFn(value), tokens);

        if (process.env.NODE_ENV !== "production") {
          styles = Object.freeze(styles);
        }

        cache.set(key, styles);
        return styles;
      };
    },

    join(...css) {
      return cls("".concat(...css));
    },

    tokens,
    themes
  });
  const DashContext = /*#__PURE__*/React.createContext({
    styles,

    setTheme(theme) {
      console.error("DashContext was consumed outside of a Provider");
    }

  });
  /**
   * A hook for consuming dash context from the provider
   */

  function useDash() {
    return React.useContext(DashContext);
  }

  const styled = function styled(Component, styles) {
    const style_ = compileStyles(styles, tokens);
    const RefForwardingComponent = /*#__PURE__*/React.forwardRef(function StyledComponent({
      style,
      ...props
    }, ref) {
      useDash();
      const propsStyle = typeof style === "function" || style === "string" ? compileStyles(style, tokens) : style;
      return /*#__PURE__*/__reactCreateElement__(Component, { ...props,
        style: propsStyle && style_ ? RN.StyleSheet.flatten([style_, propsStyle]) : propsStyle ? propsStyle : style_,
        ref: ref
      });
    }); // if (process.env.NODE_ENV !== "production") {
    //   RefForwardingComponent.displayName = `styled(${
    //     Component.displayName ?? (Component.name || "Component")
    //   })`;
    // }

    return RefForwardingComponent;
  };

  styled.ActivityIndicator = styled(RN.ActivityIndicator);
  styled.DrawerLayoutAndroid = styled(RN.DrawerLayoutAndroid);
  styled.FlatList = styled(RN.FlatList);
  styled.Image = styled(RN.Image);
  styled.ImageBackground = styled(RN.ImageBackground);
  styled.KeyboardAvoidingView = styled(RN.KeyboardAvoidingView);
  styled.Modal = styled(RN.Modal);
  styled.NavigatorIOS = styled(RN.NavigatorIOS);
  styled.RecyclerViewBackedScrollView = styled(RN.RecyclerViewBackedScrollView);
  styled.RefreshControl = styled(RN.RefreshControl);
  styled.SafeAreaView = styled(RN.SafeAreaView);
  styled.ScrollView = styled(RN.ScrollView);
  styled.SectionList = styled(RN.SectionList);
  styled.SnapshotViewIOS = styled(RN.SnapshotViewIOS);
  styled.Switch = styled(RN.Switch);
  styled.Text = styled(RN.Text);
  styled.TextInput = styled(RN.TextInput);
  styled.TouchableHighlight = styled(RN.TouchableHighlight);
  styled.TouchableNativeFeedback = styled(RN.TouchableNativeFeedback);
  styled.TouchableOpacity = styled(RN.TouchableOpacity);
  styled.TouchableWithoutFeedback = styled(RN.TouchableWithoutFeedback);
  styled.View = styled(RN.View);
  return {
    styles,
    styled,
    useDash,

    Provider({
      defaultTheme,
      children
    }) {
      const colorScheme = RN.useColorScheme();
      const [theme, setTheme] = React.useState(() => {
        if (defaultTheme) return defaultTheme;
        if (colorScheme && colorScheme in themes) return colorScheme;
        return Object.keys(themes)[0];
      });
      React.useLayoutEffect(() => {
        if (theme) tokens = mergeTokens({ ...initialTokens
        }, themes[theme]);
      }, [theme]);
      React.useLayoutEffect(() => {
        if (colorScheme && Object.values(themes).length === 2 && "light" in themes && "dark" in themes) {
          tokens = mergeTokens({ ...initialTokens
          }, themes[colorScheme]);
        }
      }, [colorScheme]);
      return /*#__PURE__*/__reactCreateElement__(DashContext.Provider, {
        value: React.useMemo(() => ({
          styles,
          theme,
          setTheme
        }), [theme]),
        children: children
      });
    }

  };
}
/**
 * A utility function that will compile style objects and callbacks into CSS strings.
 *
 * @param styles - A style callback, object, or string
 * @param tokens - A map of CSS tokens for style callbacks
 */


function compileStyles(styles, tokens) {
  const value = typeof styles === "function" ? styles(tokens) : styles;

  if (typeof value !== "string") {
    return value || {};
  }

  let cache = compileCache.get(tokens);

  if (!cache) {
    cache = new Map();
    compileCache.set(tokens, cache);
  }

  const cachedStyle = cache.get(value);
  if (cachedStyle) return cachedStyle;
  const stylePairs = [];
  const splitValue = value.split(";");

  for (let i = 0; i < splitValue.length; i++) {
    // Get prop name and prop value
    const match = propertyValuePattern.exec(splitValue[i]);
    if (match !== null) stylePairs.push([match[1], match[2]]);
  }

  try {
    const style = (0, _cssToReactNative.default)(stylePairs);
    cache.set(value, style);
    return style;
  } catch (error) {
    const msg = error.message;

    if (msg.includes("Failed to parse declaration")) {
      const values = msg.replace("Failed to parse declaration ", "").replace(/"/g, "").trim().split(":");
      console.error(`'${values[0]}' shorthand property requires units for example - ${values[0]}: 20px or ${values[0]}: 10px 20px 40px 50px`);
    }

    const style = {};
    cache.set(value, style);
    return style;
  }
}

const compileCache = /*#__PURE__*/new Map();

function mergeTokens(target, source) {
  for (const key in source) {
    var _target$key;

    const value = source[key];
    target[key] = typeof value === "object" && value !== null && !Array.isArray(value) ? mergeTokens((_target$key = target[key]) !== null && _target$key !== void 0 ? _target$key : {}, value) : value;
  }

  return target;
}

const propertyValuePattern = /\s*([^\s]+)\s*:\s*(.+?)\s*$/;

function compileLiterals(literals, ...replacements) {
  return Array.isArray(literals) ? literals.reduce((curr, next, i) => curr + next + (replacements[i] || ""), "") : literals;
}